package de.andrena.objektforum.timeconcepts;

import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.TimeZone;

import org.junit.jupiter.api.Test;

public class JavaUtilDate {

    @Test
    void now() {
        System.out.println(new Date().getTime());
        System.out.println(new Date().toInstant().toEpochMilli());
        System.out.println(Instant.now().toEpochMilli());
    }

    @Test
    void implicitSystemTimeZone() {
        Date dateFromString = new Date("Tue, 15 June 2020 16:05:00 UTC");
        Date dateFromCoordinates = new Date(120, 5, 15, 18, 5, 0);
        ZonedDateTime zonedDateTime = LocalDateTime.of(2020, 6, 15, 18, 5, 0).atZone(ZoneId.systemDefault());

        System.out.println(dateFromString);
        System.out.println(dateFromCoordinates);
        System.out.println((dateFromCoordinates.getYear() + 1900) + " " + (dateFromCoordinates.getMonth() + 1) + " " + (dateFromCoordinates.getDate()));
        System.out.println(zonedDateTime.getYear() + " " + zonedDateTime.getMonthValue() + " " + zonedDateTime.getDayOfMonth());
    }

    @Test
    void formattingDateTime() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("Asia/Singapore"));
        Date date = new Date(120, 5, 15, 18, 5, 0);
        System.out.println(simpleDateFormat.format(date) + " (with Date and SimpleDateFormat)");

        DateTimeFormatter dateTimeFormat = DateTimeFormatter
            .ofPattern("dd.MM.yyyy HH:mm")
            .withZone(ZoneId.of("Asia/Singapore"));
        ZonedDateTime dateTime = LocalDateTime.of(2020, 6, 15, 18, 5, 0).atZone(ZoneId.systemDefault());
        System.out.println(dateTimeFormat.format(dateTime) + " (with ZonedDateTime and DateTimeFormatter)");
    }

    @Test
    void formattingGlobalTimestamp() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("Asia/Singapore"));
        Date date = new Date(12345678900L);
        System.out.println(simpleDateFormat.format(date) + " (with Date and SimpleDateFormat)");

        DateTimeFormatter dateTimeFormat = DateTimeFormatter
            .ofPattern("dd.MM.yyyy HH:mm")
            .withZone(ZoneId.of("Asia/Singapore"));
        Instant instant = Instant.ofEpochMilli(12345678900L);
        System.out.println(dateTimeFormat.format(instant) + " (with Instant and DateTimeFormatter)");
    }

}
