package de.andrena.objektforum.timeconcepts;

import static java.time.Month.FEBRUARY;
import static java.time.Month.JUNE;
import static org.assertj.core.api.Assertions.assertThat;

import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

public class ZonedAndOffsetDateTime {

    @Test
    void zonedDateTime() {
        ZonedDateTime zonedDateTime = LocalDateTime
            .of(2020, JUNE, 4, 12, 42, 42)
            .atZone(ZoneId.of("Europe/Berlin"));

        System.out.println(zonedDateTime);
        System.out.println();
        System.out.println(zonedDateTime.toLocalDateTime() + " (local date time)");
        System.out.println(zonedDateTime.toInstant() + " (instant)");
    }

    @Nested
    class offsetDateTimeIgnoresWinterAndSummerTime {

        @Test
        void zoneAndOffsetMatchInWinter() {
            LocalDateTime duringWinterTime = LocalDateTime.of(2020, FEBRUARY, 4, 12, 42, 42);
            ZonedDateTime usingZoneId = duringWinterTime.atZone(ZoneId.of("Europe/Berlin"));
            OffsetDateTime usingOffset = duringWinterTime.atOffset(ZoneOffset.ofHours(1));

            System.out.println(usingZoneId);
            System.out.println(usingOffset);
            assertThat(usingZoneId.toInstant()).isEqualTo(usingOffset.toInstant());
        }

        @Test
        void zoneAndOffsetDifferInSummer() {
            LocalDateTime duringSummerTime = LocalDateTime.of(2020, JUNE, 4, 12, 42, 42);
            ZonedDateTime usingZoneId = duringSummerTime.atZone(ZoneId.of("Europe/Berlin"));
            OffsetDateTime usingOffset = duringSummerTime.atOffset(ZoneOffset.ofHours(1));

            System.out.println(usingZoneId);
            System.out.println(usingOffset);
            assertThat(usingZoneId.toInstant()).isNotEqualTo(usingOffset.toInstant());
        }
    }
}
